from aeromaps.plots.main import AirTransportCO2EmissionsPlot, AirTransportClimateImpactsPlot
from aeromaps.plots.sustainability_assessment import (
    CarbonBudgetAssessmentPlot,
    EquivalentCarbonBudgetAssessmentPlot,
    BiomassResourceBudgetAssessmentPlot,
    ElectricityResourceBudgetAssessmentPlot,
    MultidisciplinaryAssessmentPlot,
)
from aeromaps.plots.indicators import (
    MeanCO2PerRPKPlot,
    MeanCO2PerRTKPlot,
    PassengerKayaFactorsPlot,
    FreightKayaFactorsPlot,
    LeversOfActionDistributionPlot,
)
from aeromaps.plots.air_traffic import (
    RevenuePassengerKilometerPlot,
    RevenueTonneKilometerPlot,
    AvailableSeatKilometerPlot,
    TotalAircraftDistancePlot,
)
from aeromaps.plots.aircraft_fleet_and_operations import (
    DropinFuelConsumptionLiterPerPAX100kmPlot,
    MeanLoadFactorPlot,
    MeanEnergyPerASKPlot,
    MeanEnergyPerRTKPlot,
)
from aeromaps.plots.aircraft_energy import (
    MeanFuelEmissionFactorPlot,
    EmissionFactorPerFuelPlot,
    EnergyConsumptionPlot,
)
from aeromaps.plots.emissions import (
    CumulativeCO2EmissionsPlot,
    DirectH2OEmissionsPlot,
    DirectNOxEmissionsPlot,
    DirectSulfurEmissionsPlot,
    DirectSootEmissionsPlot,
)
from aeromaps.plots.climate import (
    FinalEffectiveRadiativeForcingPlot,
    DistributionEffectiveRadiativeForcingPlot,
    EquivalentEmissionsPlot,
    CumulativeEquivalentEmissionsPlot,
    EquivalentEmissionsRatioPlot,
    TemperatureIncreaseFromAirTransportPlot,
)
from aeromaps.plots.energy_resources import BiomassConsumptionPlot, ElectricityConsumptionPlot


from aeromaps.plots.costs import (
    ScenarioEnergyCapitalPlot,
    ScenarioEnergyExpensesPlot,
    ScenarioEnergyUnitCostPlot,
    DiscountEffect,
    ScenarioEnergyCarbonTaxPlot,
    ScenarioEnergyUnitCostWithCarbonTaxPlot,
    DropInMACC,
    DOCEvolutionBreakdown,
    DOCEvolutionCategory,
)

# Left plot
plot_1 = {
    "Air transport CO2 emissions": AirTransportCO2EmissionsPlot,
    "Air transport climate impact": AirTransportClimateImpactsPlot,
}

# Central plot
plot_2 = {
    "Multidisciplinary assessment": MultidisciplinaryAssessmentPlot,
    "Carbon budget assessment": CarbonBudgetAssessmentPlot,
    "Equivalent carbon budget assessment": EquivalentCarbonBudgetAssessmentPlot,
    "Biomass resource budget assessment": BiomassResourceBudgetAssessmentPlot,
    "Electricity resource budget assessment": ElectricityResourceBudgetAssessmentPlot,
}

# Right plot
plot_3 = {
    "Temperature increase from air transport": TemperatureIncreaseFromAirTransportPlot,
    "Biomass consumption": BiomassConsumptionPlot,
    "Electricity consumption": ElectricityConsumptionPlot,
    "Passenger flight emission factor (CO2 emissions per RPK)": MeanCO2PerRPKPlot,
    "Freight flight emission factor (CO2 emissions per RTK)": MeanCO2PerRTKPlot,
    "Kaya factors for passenger air transport": PassengerKayaFactorsPlot,
    "Kaya factors for freight air transport": FreightKayaFactorsPlot,
    "Distribution of levers of action impact": LeversOfActionDistributionPlot,
    "Revenue Passenger Kilometer (RPK)": RevenuePassengerKilometerPlot,
    "Revenue Tonne Kilometer (RTK)": RevenueTonneKilometerPlot,
    "Available Seat Kilometer (ASK)": AvailableSeatKilometerPlot,
    "Total distance travelled by aircraft": TotalAircraftDistancePlot,
    "Aircraft load factor": MeanLoadFactorPlot,
    "Fuel consumption (energy consumption per ASK)": MeanEnergyPerASKPlot,
    "Fuel consumption (energy consumption per RTK)": MeanEnergyPerRTKPlot,
    "Fuel consumption (fuel consumption per passenger per 100 km)": DropinFuelConsumptionLiterPerPAX100kmPlot,
    "Energy consumption of the aircraft fleet": EnergyConsumptionPlot,
    "Mean fuel emission factor (CO2 emissions per energy)": MeanFuelEmissionFactorPlot,
    "Fuel emission factors (CO2 emissions per energy)": EmissionFactorPerFuelPlot,
    "Cumulative CO2 emissions": CumulativeCO2EmissionsPlot,
    "Direct H2O emissions": DirectH2OEmissionsPlot,
    "Direct NOx emissions": DirectNOxEmissionsPlot,
    "Direct sulfur emissions": DirectSulfurEmissionsPlot,
    "Direct soot emissions": DirectSootEmissionsPlot,
    "Equivalent emissions": EquivalentEmissionsPlot,
    "Cumulative equivalent emissions": CumulativeEquivalentEmissionsPlot,
    "Equivalent emissions ratio": EquivalentEmissionsRatioPlot,
    "Effective radiative forcing in 2050": FinalEffectiveRadiativeForcingPlot,
    "Distribution of effective radiative forcing causes": DistributionEffectiveRadiativeForcingPlot,
    "Investments required per low-carbon fuel pathway": ScenarioEnergyCapitalPlot,
    "Annual expenses for energy procurement": ScenarioEnergyExpensesPlot,
    "Effect of a carbon tax on energy expenses": ScenarioEnergyCarbonTaxPlot,
    "Evolution of pathways MFSP": ScenarioEnergyUnitCostPlot,
    "Evolution of pathways MFSP, incl carbon cost": ScenarioEnergyUnitCostWithCarbonTaxPlot,
    "Effect of the discount rate on total energy costs": DiscountEffect,
    "Marginal abatement cost curve from drop in fuels": DropInMACC,
    "Direct Operating Cost breakdown": DOCEvolutionBreakdown,
    "Direct Operating Cost per aircraft category": DOCEvolutionCategory,
}



import matplotlib.pyplot as plt
import numpy as np
from .constants import plot_3_x, plot_3_y


class DropinFuelConsumptionLiterPerPAX100kmPlot:
    def __init__(self, data):
        self.df = data["vector_outputs"]
        self.float_outputs = data["float_outputs"]
        self.years = data["years"]["full_years"]
        self.historic_years = data["years"]["historic_years"]
        self.prospective_years = data["years"]["prospective_years"]

        self.fig, self.ax = plt.subplots(
            figsize=(plot_3_x, plot_3_y),
        )
        self.create_plot()

    def create_plot(self):
        self.ax.plot(
            self.historic_years,
            self.df.loc[self.historic_years, "dropin_fuel_consumption_liter_per_pax_100km"],
            color="black",
            linestyle="-",
            label="History",
            linewidth=2,
        )

        (self.line_fuel_consumption_l100km,) = self.ax.plot(
            self.prospective_years,
            self.df.loc[self.prospective_years, "dropin_fuel_consumption_liter_per_pax_100km"],
            color="blue",
            linestyle="-",
            label="Projections",
            linewidth=2,
        )

        self.ax.grid()
        self.ax.set_title(
            "Evolution of (drop-in) fuel consumption\nper passenger for air transport"
        )
        self.ax.set_xlabel("Year")
        self.ax.set_ylabel("Fuel consumption per passenger [L/100km]")
        self.ax = plt.gca()
        self.ax.legend()
        self.ax.set_xlim(2000, 2050)

        self.fig.canvas.header_visible = False
        self.fig.canvas.toolbar_position = "bottom"
        # self.fig.canvas.layout.width = "auto"
        # self.fig.canvas.layout.height = "auto"
        self.fig.tight_layout()

    def update(self, data):
        self.df = data["vector_outputs"]
        self.float_outputs = data["float_outputs"]
        self.years = data["years"]["full_years"]
        self.historic_years = data["years"]["historic_years"]
        self.prospective_years = data["years"]["prospective_years"]

        self.line_fuel_consumption_l100km.set_ydata(
            self.df.loc[self.prospective_years, "dropin_fuel_consumption_liter_per_pax_100km"]
        )

        self.ax.collections.clear()

        self.ax.relim()
        self.ax.autoscale_view()
        self.fig.canvas.draw()


class MeanLoadFactorPlot:
    def __init__(self, data):
        self.df = data["vector_outputs"]
        self.float_outputs = data["float_outputs"]
        self.years = data["years"]["full_years"]
        self.historic_years = data["years"]["historic_years"]
        self.prospective_years = data["years"]["prospective_years"]

        self.fig, self.ax = plt.subplots(
            figsize=(plot_3_x, plot_3_y),
        )
        self.create_plot()

    def create_plot(self):
        self.ax.plot(
            self.historic_years,
            self.df.loc[self.historic_years, "load_factor"],
            color="black",
            linestyle="-",
            label="History",
            linewidth=2,
        )

        (self.line_load_factor,) = self.ax.plot(
            self.prospective_years,
            self.df.loc[self.prospective_years, "load_factor"],
            color="blue",
            linestyle="-",
            label="Projections",
            linewidth=2,
        )

        self.ax.grid()
        self.ax.set_title("Evolution of the aircraft load factor")
        self.ax.set_xlabel("Year")
        self.ax.set_ylabel("Aircraft load factor [%]")
        self.ax = plt.gca()
        self.ax.legend()
        self.ax.set_xlim(2000, 2050)

        self.fig.canvas.header_visible = False
        self.fig.canvas.toolbar_position = "bottom"
        # self.fig.canvas.layout.width = "auto"
        # self.fig.canvas.layout.height = "auto"
        self.fig.tight_layout()

    def update(self, data):
        self.df = data["vector_outputs"]
        self.float_outputs = data["float_outputs"]
        self.years = data["years"]["full_years"]
        self.historic_years = data["years"]["historic_years"]
        self.prospective_years = data["years"]["prospective_years"]

        self.line_load_factor.set_ydata(self.df.loc[self.prospective_years, "load_factor"])

        self.ax.collections.clear()

        self.ax.relim()
        self.ax.autoscale_view()
        self.fig.canvas.draw()


class MeanEnergyPerASKPlot:
    def __init__(self, data):
        self.df = data["vector_outputs"]
        self.float_outputs = data["float_outputs"]
        self.years = data["years"]["full_years"]
        self.historic_years = data["years"]["historic_years"]
        self.prospective_years = data["years"]["prospective_years"]

        self.fig, self.ax = plt.subplots(
            figsize=(plot_3_x, plot_3_y),
        )
        self.create_plot()

    def create_plot(self):
        self.ax.plot(
            self.historic_years,
            self.df.loc[self.historic_years, "energy_per_ask_mean"],
            color="black",
            linestyle="-",
            label="History",
            linewidth=2,
        )

        (self.line_energy_per_ask,) = self.ax.plot(
            self.prospective_years,
            self.df.loc[self.prospective_years, "energy_per_ask_mean"],
            color="blue",
            linestyle="-",
            label="Projections",
            linewidth=2,
        )

        self.ax.grid()
        self.ax.set_title(
            "Evolution of the mean energy consumed\nper Available Seat Kilometer (ASK)"
        )
        self.ax.set_xlabel("Year")
        self.ax.set_ylabel("Energy per Available Seat Kilometer [MJ/ASK]")
        self.ax = plt.gca()
        self.ax.legend()
        self.ax.set_xlim(2000, 2050)

        self.fig.canvas.header_visible = False
        self.fig.canvas.toolbar_position = "bottom"
        # self.fig.canvas.layout.width = "auto"
        # self.fig.canvas.layout.height = "auto"
        self.fig.tight_layout()

    def update(self, data):
        self.df = data["vector_outputs"]
        self.float_outputs = data["float_outputs"]
        self.years = data["years"]["full_years"]
        self.historic_years = data["years"]["historic_years"]
        self.prospective_years = data["years"]["prospective_years"]

        self.line_energy_per_ask.set_ydata(
            self.df.loc[self.prospective_years, "energy_per_ask_mean"]
        )

        self.ax.collections.clear()

        self.ax.relim()
        self.ax.autoscale_view()
        self.fig.canvas.draw()


class MeanEnergyPerRTKPlot:
    def __init__(self, data):
        self.df = data["vector_outputs"]
        self.float_outputs = data["float_outputs"]
        self.years = data["years"]["full_years"]
        self.historic_years = data["years"]["historic_years"]
        self.prospective_years = data["years"]["prospective_years"]

        self.fig, self.ax = plt.subplots(
            figsize=(plot_3_x, plot_3_y),
        )
        self.create_plot()

    def create_plot(self):
        self.ax.plot(
            self.historic_years,
            self.df.loc[self.historic_years, "energy_per_rtk_mean"],
            color="black",
            linestyle="-",
            label="History",
            linewidth=2,
        )

        (self.line_energy_per_rtk,) = self.ax.plot(
            self.prospective_years,
            self.df.loc[self.prospective_years, "energy_per_rtk_mean"],
            color="blue",
            linestyle="-",
            label="Projections",
            linewidth=2,
        )

        self.ax.grid()
        self.ax.set_title(
            "Evolution of the mean energy consumed\nper Revenue Tonne Kilometer (RTK)"
        )
        self.ax.set_xlabel("Year")
        self.ax.set_ylabel("Energy per Revenue Tonne Kilometer [MJ/RTK]")
        self.ax = plt.gca()
        self.ax.legend()
        self.ax.set_xlim(2000, 2050)

        self.fig.canvas.header_visible = False
        self.fig.canvas.toolbar_position = "bottom"
        # self.fig.canvas.layout.width = "auto"
        # self.fig.canvas.layout.height = "auto"
        self.fig.tight_layout()

    def update(self, data):
        self.df = data["vector_outputs"]
        self.float_outputs = data["float_outputs"]
        self.years = data["years"]["full_years"]
        self.historic_years = data["years"]["historic_years"]
        self.prospective_years = data["years"]["prospective_years"]

        self.line_energy_per_rtk.set_ydata(
            self.df.loc[self.prospective_years, "energy_per_rtk_mean"]
        )

        self.ax.collections.clear()

        self.ax.relim()
        self.ax.autoscale_view()
        self.fig.canvas.draw()



import matplotlib.pyplot as plt
import numpy as np
from .constants import plot_3_x, plot_3_y


class BiomassConsumptionPlot:
    def __init__(self, data):
        self.df = data["vector_outputs"]
        self.float_outputs = data["float_outputs"]
        self.years = data["years"]["full_years"]
        self.historic_years = data["years"]["historic_years"]
        self.prospective_years = data["years"]["prospective_years"]

        self.fig, self.ax = plt.subplots(
            figsize=(plot_3_x, plot_3_y),
        )
        self.create_plot()

    def create_plot(self):
        (self.line_biomass_consumption,) = self.ax.plot(
            self.prospective_years,
            self.df.loc[self.prospective_years, "biomass_consumption"],
            color="blue",
            linestyle="-",
            label="Aviation biomass consumption",
            linewidth=2,
        )

        self.ax.grid()
        self.ax.set_title("Evolution of biomass consumption\nfrom air transport")
        self.ax.set_xlabel("Year")
        self.ax.set_ylabel("Biomass consumption [EJ]")
        self.ax = plt.gca()
        self.ax.legend()
        self.ax.set_xlim(2020, 2050)

        self.fig.canvas.header_visible = False
        self.fig.canvas.toolbar_position = "bottom"
        # self.fig.canvas.layout.width = "auto"
        # self.fig.canvas.layout.height = "auto"
        self.fig.tight_layout()

    def update(self, data):
        self.df = data["vector_outputs"]
        self.float_outputs = data["float_outputs"]
        self.years = data["years"]["full_years"]
        self.historic_years = data["years"]["historic_years"]
        self.prospective_years = data["years"]["prospective_years"]

        self.line_biomass_consumption.set_ydata(
            self.df.loc[self.prospective_years, "biomass_consumption"]
        )

        self.ax.collections.clear()

        self.ax.relim()
        self.ax.autoscale_view()
        self.fig.canvas.draw()


class ElectricityConsumptionPlot:
    def __init__(self, data):
        self.df = data["vector_outputs"]
        self.float_outputs = data["float_outputs"]
        self.years = data["years"]["full_years"]
        self.historic_years = data["years"]["historic_years"]
        self.prospective_years = data["years"]["prospective_years"]

        self.fig, self.ax = plt.subplots(
            figsize=(plot_3_x, plot_3_y),
        )
        self.create_plot()

    def create_plot(self):

        (self.line_electricity_consumption,) = self.ax.plot(
            self.prospective_years,
            self.df.loc[self.prospective_years, "electricity_consumption"],
            color="blue",
            linestyle="-",
            label="Aviation electricity consumption",
            linewidth=2,
        )

        (self.line_france_production_2019,) = self.ax.plot(
            self.prospective_years,
            np.ones(len(self.prospective_years)) * 537.7 * 3.6 / 1000,
            color="black",
            linestyle="--",
            label="France electricity production (2019)",
            linewidth=1,
        )

        self.ax.grid()
        self.ax.set_title("Evolution of electricity consumption\nfrom air transport")
        self.ax.set_xlabel("Year")
        self.ax.set_ylabel("Electricity consumption [EJ]")
        self.ax = plt.gca()
        self.ax.legend()
        self.ax.set_xlim(2020, 2050)

        self.ax_twin = self.ax.twinx()
        self.ax_twin.set_ylabel("Electricity consumption [TWh]")
        function_ej_to_twh = lambda elec: elec * 1000 / 3.6
        ymin, ymax = self.ax.get_ylim()
        self.ax_twin.set_ylim((function_ej_to_twh(ymin), function_ej_to_twh(ymax)))
        self.ax_twin.plot([], [])

        self.fig.canvas.header_visible = False
        self.fig.canvas.toolbar_position = "bottom"
        # self.fig.canvas.layout.width = "auto"
        # self.fig.canvas.layout.height = "auto"
        self.fig.tight_layout()

    def update(self, data):
        self.df = data["vector_outputs"]
        self.float_outputs = data["float_outputs"]
        self.years = data["years"]["full_years"]
        self.historic_years = data["years"]["historic_years"]
        self.prospective_years = data["years"]["prospective_years"]

        self.line_electricity_consumption.set_ydata(
            self.df.loc[self.prospective_years, "electricity_consumption"]
        )

        self.line_france_production_2019.set_ydata(
            np.ones(len(self.prospective_years)) * 537.7 * 3.6 / 1000
        )

        self.ax.collections.clear()

        self.ax.relim()
        self.ax.autoscale_view()
        self.ax_twin.relim()
        self.ax_twin.autoscale_view()
        self.fig.canvas.draw()
